/* eslint-env jest */

const { runPlugins } = require("./helpers");

const mixinPlugin_ = require("../mixin");
const mixinPlugin = x => mixinPlugin_(x, { transform: x => `token(${x})` });

it("ignores optional args that are invalid tokens", () => {
  let actual = runPlugins(
    `
    @mixin my-mixin-a ($a: $color-brand) {}
  `,
    [mixinPlugin([])]
  );
  expect(actual).toMatchSnapshot();
});

it("tokenizes optional args that are valid tokens", () => {
  let actual = runPlugins(
    `
    @mixin my-mixin-a ($a: $color-brand) {}
  `,
    [mixinPlugin(["color-brand"])]
  );
  expect(actual).toMatchSnapshot();
});

it("ignores optional args that are non-tokenizable", () => {
  let actual = runPlugins(
    `
    @mixin my-mixin-a ($a: $color-brand) {
      color: rgba($a);
    }
  `,
    [mixinPlugin(["color-brand"])]
  );
  expect(actual).toMatchSnapshot();
});

it("ignores optional args that are non-tokenizable (multiple instances)", () => {
  let actual = runPlugins(
    `
    @mixin my-mixin-a ($a: $color-brand) {
      color: $a; // tokenizable
      background-color: rgba($a); // non-tokenizable
    }
  `,
    [mixinPlugin(["color-brand"])]
  );
  expect(actual).toMatchSnapshot();
});

it("ignores optional args that are non-tokenizable because the include mixin arg is non-tokenizable", () => {
  let actual = runPlugins(
    `
    @mixin my-mixin-a ($a) {
      color: rgba($a);
    }
    @mixin my-mixin-b ($b: $color-brand) {
      color: $b;
      @include my-mixin-a($b);
    }
  `,
    [mixinPlugin(["color-brand"])]
  );
  expect(actual).toMatchSnapshot();
});

it("ignores optional args that are non-tokenizable because the usage in an @include is non-tokenizable (args)", () => {
  let actual = runPlugins(
    `
    @mixin my-mixin-a ($a) {
      color: $a;
    }
    @mixin my-mixin-b ($b: $color-brand) {
      color: $b;
      @include my-mixin-a(rgba($b));
    }
  `,
    [mixinPlugin(["color-brand"])]
  );
  expect(actual).toMatchSnapshot();
});

it("ignores optional args that are non-tokenizable because the usage in an @include is non-tokenizable (parens)", () => {
  let actual = runPlugins(
    `
    @mixin my-mixin-a ($a) {
      color: $a;
    }
    @mixin my-mixin-b ($b: $color-brand) {
      color: $b;
      @include my-mixin-a(($b));
    }
  `,
    [mixinPlugin(["color-brand"])]
  );
  expect(actual).toMatchSnapshot();
});

it("ignores optional args that are non-tokenizable because the usage in an @include is non-tokenizable (interpolation)", () => {
  let actual = runPlugins(
    `
    @mixin my-mixin-a ($a) {
      color: $a;
    }
    @mixin my-mixin-b ($b: $color-brand) {
      color: $b;
      @include my-mixin-a(#{$b});
    }
  `,
    [mixinPlugin(["color-brand"])]
  );
  expect(actual).toMatchSnapshot();
});

it("tokenizes optional args based on useage as a tokenizable @include arg", () => {
  let actual = runPlugins(
    `
    @mixin my-mixin-a ($a) {
      color: $a;
    }
    @mixin my-mixin-b ($b: $color-brand) {
      color: $b;
      @include my-mixin-a($b);
    }
  `,
    [mixinPlugin(["color-brand"])]
  );
  expect(actual).toMatchSnapshot();
});

it("ignores @include args that are non-tokenizable", () => {
  let actual = runPlugins(
    `
    @mixin my-mixin-b ($b: $color-brand) {
      color: ($b);
    }
    @include my-mixin-b($color-brand);
  `,
    [mixinPlugin(["color-brand"])]
  );
  expect(actual).toMatchSnapshot();
});

it("tokenizes @include args that are tokenizable", () => {
  let actual = runPlugins(
    `
    @mixin my-mixin-b ($b) {
      color: $b;
    }
    @include my-mixin-b($color-brand);
  `,
    [mixinPlugin(["color-brand"])]
  );
  expect(actual).toMatchSnapshot();
});

it("throws an error for undefined mixins", () => {
  expect(() =>
    runPlugins(`@include my-mixin($color-brand);`, [
      mixinPlugin(["color-brand"])
    ])
  ).toThrow(`Undefined mixin "my-mixin"`);
});

it("collects mixins with no arguments", () => {
  expect(() =>
    runPlugins(
      `
      @mixin my-mixin {}
      @include my-mixin($color-brand);
    `,
      [mixinPlugin(["color-brand"])]
    )
  ).not.toThrow();
});
